from modules.parameters import ParameterSpace, copy_dict
from modules.signals import empty
from modules.visualization import *
from modules.io import set_project_paths
from defaults.paths import paths
import matplotlib.pyplot as pl
from matplotlib import cm
from os import environ, system
from scipy.interpolate import SmoothBivariateSpline, Rbf, griddata
import sys

"""
stimulus_processing
- read data recorded from stimulus_processing experiments
"""

# data parameters
project = 'encoding_decoding'
data_type = 'dcinput'#'spikepatterninput' #
#population_of_interest = 'E'  # results are provided for only one population (choose Global to get the totals)
#data_path = '/home/neuro/Desktop/MANUSCRIPTS/in_preparation/Encoding_Decoding/data/training_parameters/'
data_path = '/media/neuro/Data/EncodingDecoding_NEW/training_parameters/'
data_label = 'ED_{0}_training_parameters'.format(data_type)  # read trial0 to extract data structure
results_path = data_path + data_label + '/Results/'

# set defaults and paths
set_project_paths(project)
set_global_rcParams(paths['local']['matplotlib_rc'])

# re-create ParameterSpace
pars_file = data_path + data_label + '_ParameterSpace.py'
pars = ParameterSpace(pars_file)

# print the full nested structure of the results dictionaries (to know where are the results of interest and how to
# harvest them)
pars.print_stored_keys(results_path)

# harvest the whole data set - returns a tuple with the data labels and the complete results dictionaries
# results = pars.harvest(data_path+data_label+'/Results/')

#################
# Analyse and plot, harvesting only the specified result (specifying the nested sequence of dictionary keys necessary
#  to reach the data - key_sets)
#################
# global plot properties
pl_props = copy_dict(pars.parameter_axes, {'xlabel': r'$' + pars.parameter_axes['ylabel'] + '$',
                                           'ylabel': r'$' + pars.parameter_axes['xlabel'] + '$',
                                           'xticklabels': pars.parameter_axes['yticklabels'],
                                           'yticklabels': pars.parameter_axes['xticklabels'][::4],
                                           'xticks': np.arange(0., len(pars.parameter_axes['yticks']), 1.),
                                           'yticks': np.arange(0., len(pars.parameter_axes['xticks']), 4.),})
analysis = {
	'title': 'Pinv_classifier',
	'variable_names': [r'$Accuracy$'],#, r'$rawMSE$', r'$maxMSE$', r'$|W_{\mathrm{out}}|$'],
	'key_sets': ['performance/EI/V_m0/pinv_classifier/label/performance'],}
	             # 'performance/EI/V_m0/pinv_classifier/raw/MSE',
	             # 'performance/EI/V_m0/pinv_classifier/max/MSE',
	             # 'performance/EI/V_m0/pinv_classifier/norm_wOut',],}
##
fig = pl.figure(figsize=(4, 3))
fig.suptitle(analysis['title'])
n_subplots = len(analysis['variable_names'])
axes = []
arrays = []
arrays_interp = []
remove_indices = [(1, 0), (3, 1), (5, 2), (7, 3), (9, 4), (15, 7), (11, 5), (17, 8)] # incorrect results
for idx, var in enumerate(analysis['variable_names']):
	globals()['ax{0}'.format(idx)] = fig.add_subplot(int(np.floor(np.sqrt(n_subplots))), int(np.ceil(np.sqrt(
		n_subplots))), idx+1)
	array = pars.harvest(results_path, key_set=analysis['key_sets'][idx])[1]
	for indice in remove_indices:
		array[indice] = np.nan

	# try interpolation (to fill the missing data)
	x = pars.parameter_axes['xticks']
	y = pars.parameter_axes['yticks']
	z = array.astype(float)

	nx, ny = len(x), len(y)
	xi, yi = np.meshgrid(np.linspace(x.min(), x.max(), nx),
	                     np.linspace(y.min(), y.max(), ny))

	point_coordinates = [(k, v) for k in x for v in y]
	x_vals = np.array([xx[0] for xx in point_coordinates])
	y_vals = np.array([xx[1] for xx in point_coordinates])
	z_vals = z.flatten()

	points = np.array([(k, v) for k in x for v in y if not np.isnan(z[np.where(x==k)[0][0], np.where(y==v)[0][0]])])
	values = np.array([z[np.where(x==k)[0][0], np.where(y==v)[0][0]] for k in x for v in y if not np.isnan(z[
		                                                                                                       np.where(x==k)[0][0], np.where(y==v)[0][0]])])

	grid_z = griddata(points, values, (xi, yi), method='nearest')

	# ax.scatter3D(x_vals, y_vals, z_vals, c=z_vals, cmap=cm.winter)
	# surf = ax.plot_surface(xi, yi, grid_z, cmap='winter', linewidth=1, alpha=0.8, rstride=1, cstride=1,
	#                                               antialiased=True)

	arrays_interp.append(grid_z.astype(float).T)
	axes.append(globals()['ax{0}'.format(idx)])
	arrays.append(array.astype(float))
# plot_2d_parscans(arrays, axis=axes, fig_handle=fig, labels=analysis['variable_names'], **pl_props)
plot_2d_parscans(arrays_interp, axis=axes, fig_handle=fig, labels=analysis['variable_names'], **pl_props)
for ax in axes:
	ax.get_images()[0].set_clim(0, 1)

print np.sum(arrays_interp[0])/arrays_interp[0].size
pl.show()

analysis2 = {
	'title': 'Ridge classifier',
	'variable_names': [r'$Accuracy$', r'$rawMSE$', r'$maxMSE$', r'$|W_{\mathrm{out}}|$'],
	'key_sets': ['performance/EI/V_m0/ridge_classifier/label/performance',
				 'performance/EI/V_m0/ridge_classifier/raw/MSE',
	             'performance/EI/V_m0/ridge_classifier/max/MSE',
	             'performance/EI/V_m0/ridge_classifier/norm_wOut',],}
##
fig2 = pl.figure()
fig2.suptitle(analysis2['title'])
n_subplots = len(analysis2['variable_names'])
axes = []
arrays = []
for idx, var in enumerate(analysis2['variable_names']):
	globals()['ax{0}'.format(idx)] = fig2.add_subplot(int(np.floor(np.sqrt(n_subplots))), int(np.ceil(np.sqrt(
		n_subplots))), idx+1)
	array = pars.harvest(results_path, key_set=analysis2['key_sets'][idx])[1]

	axes.append(globals()['ax{0}'.format(idx)])
	arrays.append(array.astype(float))

plot_2d_parscans(arrays, axis=axes, fig_handle=fig2, labels=analysis2['variable_names'], **pl_props)

analysis3 = {
	'title': 'Dimensionality',
	'variable_names': [r'$Dimensionality$'],
	'key_sets': ['dimensionality/EI/V_m0'],}
##
fig3 = pl.figure()
fig3.suptitle(analysis3['title'])
n_subplots = len(analysis3['variable_names'])
axes = []
arrays = []
for idx, var in enumerate(analysis3['variable_names']):
	globals()['ax{0}'.format(idx)] = fig3.add_subplot(int(np.floor(np.sqrt(n_subplots))), int(np.ceil(np.sqrt(
		n_subplots))), idx+1)
	array = pars.harvest(results_path, key_set=analysis3['key_sets'][idx])[1]

	axes.append(globals()['ax{0}'.format(idx)])
	arrays.append(array.astype(float))

plot_2d_parscans(arrays, axis=axes, fig_handle=fig3, labels=analysis3['variable_names'], **pl_props)


pl.show()